@@ -45,11 +45,21 @@
 #include <cc++/config.h>
 #endif
 
+#ifndef	CCXX_THREAD_H_
+#include <cc++/thread.h>
+#endif
+
 #ifdef	CCXX_NAMESPACES
 namespace ost {
 #endif
 
 #define	AUDIO_SIGNED_LINEAR_RAW	1
+#define	AUDIO_LINEAR_CONVERSION 1
+#define	AUDIO_CODEC_MODULES	1
+
+typedef	signed short *Linear;
+
+class AudioCodec;
 
 /**
  * Generic audio class to hold master data types and various useful
@@ -129,7 +139,8 @@ public:
 		errStatFailed,
 		errInvalidTrack,
 		errPlaybackFailed,
-		errNotPlaying
+		errNotPlaying,
+		errNoCodec
 	};
 	typedef enum Error Error;
 
@@ -224,6 +235,8 @@ protected:
 	virtual bool afOpen(const char *path);
 	virtual bool afPeek(unsigned char *data, unsigned size);
 
+	AudioCodec *getCodec(void);
+
 	/**
 	 * Read a given number of bytes from the file, starting from
 	 * the current file pointer.  May be overridden by derived
@@ -344,6 +357,16 @@ public:
 	 */
 	int getBuffer(void *addr, unsigned len);
 
+	/**
+	 * Retrieve and convert content to linear encoded audio data
+	 * from it's original form.
+	 *
+	 * @param addr A pointer to copy linear data into.
+	 * @param len Number of linear samples to extract.
+	 * @return number of samples retrieved, 0 if no codec or eof.
+	 */
+	unsigned getLinear(Linear buffer, unsigned request);
+
 	/**
 	 * Insert bytes into the file from a memory buffer.  This
 	 * increments the file pointer so subsequent calls append
@@ -358,6 +381,16 @@ public:
 	 */
 	int putBuffer(void *attr, unsigned len);
 
+        /**
+         * Convert and store content from linear encoded audio data
+         * to the format of the audio file.
+         *
+         * @param addr A pointer to copy linear data from.
+         * @param len Number of linear samples to save.
+         * @return number of samples saved, 0 if no codec or eof.
+         */
+        unsigned putLinear(Linear buffer, unsigned request);
+
 	/**
 	 * Retrieve samples from the file into a memory buffer.  This
 	 * increments the file pointer so subsequent calls read
@@ -439,6 +472,13 @@ public:
 
 	inline bool operator!(void)
 		{return (bool)!isOpen();};
+
+	/**
+	 * Return if the current content is signed or unsigned samples.
+	 *
+	 * @return true if signed.
+	 */
+	bool isSigned(void);
 };
 
 /**
@@ -534,6 +574,21 @@ public:
 		{return samples;};
 };
 
+/**
+ * The linear class is a type variation of AudioSample used specifically 
+ * to tag audio conversions that are performed between "linear" and
+ * some other encoding.
+ */
+class CCXX_CLASS_EXPORT LinearSample : public AudioSample
+{
+public:
+	LinearSample(unsigned frame, unsigned rate = 8000) :
+	AudioSample(frame, pcm16Mono, rate) {};
+
+	inline short *getSamples(void)
+		{return (short *)samples;};
+};
+
 /**
  * The codec class is a virtual used for transcoding audio samples between
  * linear frames (or other known format) and an encoded "sample" buffer.
@@ -546,205 +601,41 @@ public:
  * @author David Sugar <dyfet@ostel.com>
  * @short process codec interface.
  */
-class CCXX_CLASS_EXPORT AudioCodec : public AudioSample
+class CCXX_CLASS_EXPORT AudioCodec : public Audio
 {
 protected:
-	AudioCodec(unsigned frames, Encoding encoding, unsigned rate);
-	virtual ~AudioCodec() {};
-	unsigned length;
+	static Mutex lock;
+	static AudioCodec *first;
+	AudioCodec *next;
+	Encoding encoding;
+	const char *name;
 
 public:
+	AudioCodec(const char *n, Encoding e);
+	virtual ~AudioCodec() {};
+
+	static AudioCodec *getCodec(Encoding encoding, const char *name = NULL);
+	static bool load(const char *name);
+
 	/**
 	 * Encode a linear sample frame into the codec sample buffer.
 	 *
-	 * @param buffer linear sample buffer to encode.
-	 * @param samples The number of samples, or use default count.
-	 * @param channel The channel number, or 0 for all channels.
+	 * @param number of bytes written.
+	 * @param buffer linear sample buffer to use.
+	 * @param dest buffer to store encoded results.
+	 * @param lsamples The number of linear samples.
 	 */
-	virtual void encode(unsigned short *buffer, unsigned samples = 0, int channel = 0) = 0;
-
-	/**
-	 * Encode an "AudioSamples" object into the new encoding scheme if
-	 * conversion is supported.
-	 *
-	 * @return true if successful.
-	 * @param sample object source.
-	 */
-	virtual bool encode(AudioSample *sample) = 0;
+	virtual unsigned encode(Linear buffer, void *dest, unsigned lsamples) = 0;
 
 	/**
 	 * Decode the sample frame into linear samples.
 	 *
-	 * @return number of samples extracted.
+	 * @return number of bytes scanned.
 	 * @param buffer sample buffer to save linear samples into.
-	 * @param channel channel number or 0 for all channels.
+	 * @param source for encoded data.
+	 * @param number of samples to extract.
 	 */
-	virtual unsigned decode(unsigned short *buffer, int channel = 0) = 0;
-
-	/**
-	 * Decode into an :AudioSample" if the conversion is supported.
-	 *
- 	 * @return true if successful.
-	 * @param sample object to save into.
-	 */
-	virtual bool decode(AudioSample *sample) = 0;
-
-	/**
-	 * Load a codec from a byte stream data source.  This is used for
-	 * streaming of compressed codec's where frames are of irregular
-	 * sizes.
-	 *
-	 * @return number of bytes actually read into codec workspace.
-	 * @param data pointer to byte data stream.
-	 * @param length number of data bytes.
-	 */
-	virtual unsigned load(unsigned char *data, unsigned length);
-
-	/**
-	 * Save a codec to a byte stream data source.  This is used for
-	 * streaming of compressed codec's where frames are of irregular
-	 * sizes.
-	 *
-	 * @return number of bytes actually written to workspace.
-	 * @param data pointer to byte data stream.
-	 * @param length size of byte data stream.
-	 */
-	virtual unsigned save(unsigned char *data, unsigned length);
-
-
-	/**
-	 * Get current number of samples of data, based on load or encode.
-	 *
-	 * @return length.
-	 */
-	inline unsigned getLength(void)
-		{return length;};
-
-	/**
-	 * Get size of the audio frame.
-	 */
-	unsigned getFrameSize(void);
-
-	/**
-	 * Check if stereo/multi-channel codec.
-	 */
-	virtual unsigned getChannels(void);
-};
-
-/**
- * This class is used by loadable modules and application specific code to
- * register new codec modules.  A derived xxxRegisterCodec class will
- * typically exist as a static member of an xxxCodec class to
- * automatically register the codec at system load time.
- *
- * @author David Sugar <dyfet@ostel.com>
- * @short a base class for codec registration.
- */
-class CCXX_CLASS_EXPORT AudioRegisterCodec : public Audio
-{
-private:
-	static AudioRegisterCodec *first;
-	AudioRegisterCodec *next;
-
-protected:
-	unsigned minframe, framecnt;
-	AudioRegisterCodec(unsigned min, unsigned def = 1);
-
-public:
-	/**
-	 * Get the encoding format of this codec registry.
-	 *
-	 * @return audio encoding format.
-	 */
-	virtual Encoding getEncoding(void) = 0;
-
-	/**
-	 * Get the default sample rate for this codec type.
-	 *
-	 * @return samples per second.
-	 */
-	virtual unsigned getRate(void)
-		{return Audio::getRate(getEncoding());};
-
-	/**
-	 * Get a logical name for this codec.
-	 *
-	 * @return null terminated name string.
-	 */
-	virtual char *getName(void) = 0;
-
-	/**
-	 * Creates a new codec from the registry class.  This creates
-	 * an object of "frame" size, as a multiple of the min. frame
-	 * size.  This is usually just a new xxxCodec(frames * minframe).
-	 *
-	 * @param frames number of "minframe" sample frames in transcoder.
-	 */
-	virtual AudioCodec *getCodec(unsigned frames) = 0;
-
-	/**
-	 * Create a new Tone object from the registry class.  This
-	 * creates a tone buffer (AudioTone-like sample) encoded in
-	 * the format of the given codec.
-	 *
-	 * @param f1 first tone frequency.
-	 * @param f2 second tone frequency.
-	 * @param maxsize max frames of sample.
-         * @param maxtone max frames of tone.
-	 * @return audio sample in encoded format.
-	 */
-	virtual AudioSample *getTone(unsigned f1, unsigned f2, unsigned maxsize, unsigned maxtone = 0) = 0;
-
-	/**
-	 * Return minimum codec frame size in samples.
-	 *
-	 * @return codec transcoding sample size resolution.
-	 */
-	inline unsigned getFrameSize(void)
-		{return minframe;};
-
-	/**
-	 * Get the default or preferred number of frames for optimal
-	 * performance.
-	 *
-	 * @return codec transcoding preferred sample resolution.
-	 */
-	inline unsigned getFrameCount(void)
-		{return framecnt;};
-
-	/**
-	 * Locate a codec registry object.
-	 *
-	 * @param encoding encoding of codec registry to locate.
-	 * @param rate optional sample rate of codec registry to locate.
-	 */
-	static AudioRegisterCodec *findRegistration(Encoding encoding, unsigned rate = 0);
-
-	/**
-	 * Locate a codec registry object by name.
-	 *
-	 * @param name name of codec registry to locate.
-	 */
-	static AudioRegisterCodec *findRegistration(const char *name);
-
-	/**
-	 * Create a new codec object from the registry to use for
-	 * transcoding.
-	 *
-	 * @param encoding encoding of codec to create.
-	 * @param rate optional sample rate of codec being created.
-	 * @param frames sample frames (multiples of minframe) or 0 for defframe.
-	 */
-	static AudioCodec *findCodec(Encoding encoding, unsigned rate = 0, unsigned frames = 0);
-
-	/**
-	 * Create a new codec object from a named registry to use for
-	 * transcoding.
-	 *
-	 * @param name name of codec to create.
-	 * @param frame frame count to process or 0 for default frame size.
-	 */
-	static AudioCodec *findCodec(const char *name, unsigned frame = 0);
+	virtual unsigned decode(Linear buffer, void *source, unsigned lsamples) = 0;
 };
 
 /**
